package Status;
use strict;
use Socket;
use Sys::Hostname;

sub new {
    my ($class_name) = shift;
    my $self = {
        DBNAME => shift,
        DB     => shift,
        TABLE  => shift,
    };
    bless( $self, $class_name );
    $self->{_created} = 1;
    return $self;
}

sub database {
    my $self = shift;
    if (@_) { $self->{DBNAME} = shift; }
    return $self->{DBNAME};
}

sub db {
    my $self = shift;
    if (@_) { $self->{DB} = shift; }
    return $self->{DB};
}

sub table {
    my $self = shift;
    if (@_) { $self->{TABLE} = shift; }
    return $self->{TABLE};
}

sub openFile {
    my ( $self, $filename, $cond ) = @_;
    my $file;
    my $read  = 0;
    my $write = 0;

    # Read or Write conditions
    if ( $cond =~ /read/ ) {
        $read = 1;
    }
    elsif ( $cond =~ /write/ ) {
        $write = 1;
    }

    # Open file with traps sent
    if ( $filename && $read ) {
	print "$filename\n";
        open( $file, "<$filename" ) or die "Can't open file $filename";
    }
    elsif ( $filename && $write ) {
        open( $file, ">$filename" ) or die "Can't open file $filename";
    }
    else {
        print "Usage: ./trapd.pl FILE\n";
        return 0;
    }
    return $file;
}

# Load the file of traps sent in argv
sub loadStatus {
    my ( $self, $line, $date, $time, $routerName, $ip ) = @_;
    my $status = "";
    if ( $line =~ /linkDown/ ) {
        print "$date $time - $routerName ($ip) - ";
        $line =~ /STRING:[\s]+?([\w]+\/[\w]+).*STRING:[\s]+?(.*)/;
        print "$1 - $2 - DOWN\n";
        $status = "down";
    }
    elsif ( $line =~ /linkUp/ ) {
        print "$date $time - $routerName ($ip) - ";
        $line =~ /STRING:[\s]+?([\w]+\/[\w]+).*STRING:[\s]+?(.*)/;
        print "$1 - $2 - UP\n";
        $status = "up";
    }
    return $status;
}

# This function gets the file of traps and inserts in the BD the status of interfaces and additional info of them
sub runTrapGetter {
    my ( $self, $filename ) = @_;
    my $cvar = "";
    my $file = $self->openFile( $filename, "read" );

    while (<$file>) {
        my $date        = "";    # Date with format DD-MM-YYYY
        my $reverseDate = "";    # YYYY-MM-DD
        my $time        = "";
        my $ip          = "";
        my $routerName  = "";
        my $status      = "";    # Down or Up
        my $port        = "";    # FastEthernet, GigabitEthernet etc

        # Regex of the trap file
        if ( $_ =~
/([\d]+)-([\d]+)-([\d]+)[\s]+?([\d]+:[\d]+)[\s]+?from.*[\s]+?(.*)[\s]+?\(([\d]+\.[\d]+\.[\d]+\.[\d]+)\).*IF-MIB::ifDescr.[\d]+[\s]+?=[\s]+?STRING:[\s]+?([\w]+\/[\d]+)/
          )
        {
            $date        = $1 . "-" . $2 . "-" . $3;
            $reverseDate = $3 . "-" . $2 . "-" . $1;
            $time        = $4;
            $routerName  = $5;
            $port        = $7;
            if ( $routerName =~ /[\d]+\.[\d]+\.[\d]+\.[\d]+/ ) {
                $ip = $routerName;
            }
            else {
                $ip = inet_ntoa( scalar( gethostbyname($routerName) ) );
            }

            $status = $self->loadStatus( $_, $date, $time, $routerName, $ip );
            if ( $status =~ "down" || $status =~ "up" ) {
                my $sqlinsert = "
                                INSERT IGNORE INTO " . $self->table() . "
                                (DATE, TIME, ROUTER, IP, INTERFACE, STATUS)
                                VALUES('$reverseDate','$time','$routerName', INET_ATON('$ip'), '$port', '$status')";
                $cvar = $self->db()->prepare($sqlinsert);
                $cvar->execute()
                  or die DBI->errstr;
            }
        }
    }
    $cvar->finish();

    $file = $self->openFile($filename, "write"); # Blanks the file to restart the collect of traps
    close($file);
    return 1;
}

# Plot the data in status.data with Gnuplot
sub runPlotter {
    my (
        $self,      $filename,   $maxHostDown, $initialDate,
        $finalDate, $routerName, $interval,    $xtics,
        $ytics,     $plotter,    $plotOutput
    ) = @_;
    my $xMinRange = 0;
    my $xMaxRange = 0;
    my $yMaxRange = 0;
    my $dateRange = 0;
    my $timefmt   = "";
    my $formatx   = "";
    my $boxwidth  = 0;
    my $size      = "1.8,1.5";

    if ( $initialDate && $finalDate ) {

        # This defines de ranges for the x in graph
        if ( $interval =~ /Daily/ ) {
            my $dayRange = 3;
            $initialDate =~ /([\d]+)-([\d]+)-([\d]+)/;
            $dateRange = $3 - $dayRange;
            $xMinRange = $1 . "-" . $2 . "-" . $dateRange;
            $finalDate =~ /([\d]+)-([\d]+)-([\d]+)/;
            $dateRange = $3 + $dayRange;
            $xMaxRange = $1 . "-" . $2 . "-" . $dateRange;
            $timefmt   = "%Y-%m-%d";
            $formatx   = "%d/%m";
            $boxwidth  = 50000;
        }
        elsif ( $interval =~ /Monthly/ ) {
            my $monthRange = 5;
            $initialDate =~ /([\d]+)-([\d]+)-[\d]+/;
            $dateRange = $2 - $monthRange;
            $xMinRange = $1 . "-" . $dateRange;
            $finalDate =~ /([\d]+)-([\d]+)-[\d]+/;
            $dateRange = $2 + $monthRange;
            $xMaxRange = $1 . "-" . $dateRange;
            $timefmt   = "%Y-%m";
            $formatx   = "%m/%Y";
            $boxwidth  = 1500000;
        }
        elsif ( $interval =~ /Yearly/ ) {
            my $yearRange = 5;
            $initialDate =~ /([\d]+)-[\d]+-[\d]+/;
            $dateRange = $1 - $yearRange;
            $xMinRange = $dateRange;
            $finalDate =~ /([\d]+)-[\d]+-[\d]+/;
            $dateRange = $1 + $yearRange;
            $xMaxRange = $dateRange;
            $timefmt   = "%Y";
            $formatx   = "%Y";
            $boxwidth  = 14000000;
        }

        # This defines de ranges for the y in graph
        $yMaxRange = $maxHostDown + 1;

        # Plot of status data
        open( GNUPLOT, "| $plotter" );
        print GNUPLOT <<EOPLOT;
		set boxwidth $boxwidth
		set term png small xFFFFFF
		set output "$plotOutput"
		set data style line
		set xlabel "Date"
		set ylabel "Absolute Incidents"
		set title "Total $interval Incidents Sent by $routerName"
		set grid xtics ytics 
		set xdata time
		set timefmt "$timefmt"
		set format x "$formatx"
		set size $size
		set xrange ["$xMinRange":"$xMaxRange"]
		set yrange [0:$yMaxRange]
		set xtics ($xtics);
		set ytics ($ytics);
		plot "$filename" using 1:2 notitle w boxes fs solid 0.4 
EOPLOT
        close(GNUPLOT);
        return 1;
    }
    return 0;
}

sub applyDate {
    my ( $self, $source, $interval ) = @_;
    my $date = "";

    # Strings for comparison with interval
    my $daily   = "Daily";
    my $monthly = "Monthly";
    my $yearly  = "Yearly";

    if ( $interval =~ /$daily/ ) {
        $source =~ /([\d]+-[\d]+-[\d]+)/;
        $date = $1;
    }
    elsif ( $interval =~ /$monthly/ ) {
        $source =~ /([\d]+-[\d]+)-[\d]+/;
        $date = $1;
    }
    elsif ( $interval =~ /$yearly/ ) {
        $source =~ /([\d]+)-[\d]+-[\d]+/;
        $date = $1;
    }
    return $date;
}

# This gathers the data of the table of informations and create the file status.data for plotting
sub runDataGather {
    my ( $self, $filename, $routerName, $interval, $plotter, $plotOutput,
        @table )
      = @_;
    my $hostDown       = 0;
    my $maxHostDown    = 0;
    my $totalIncidents = 0;
    my $prevDate       = "";
    my $currentDate    = "";
    my $finalDate      = "";
    my @xtics          = ();
    my @ytics          = ();

    # Number of columns of query
    my $dateCol   = 0;    # Change it for the date column of the query
    my $timeCol   = 1;    # Change it for the time column of the query
    my $statusCol = 2;    # Change it for the status column of the query
    my $ifCol     = 3;    # Interface column of the query

    # String for down and up
    my $down = "down";
    my $up   = "up";

    $prevDate = $self->applyDate( $table[0][$dateCol], $interval );
    open( FILE, ">$filename" );
    for ( my $row = 0 ; $row < @table ; $row++ ) {
        $currentDate = $self->applyDate( $table[$row][$dateCol], $interval );
        if ( $prevDate !~ $currentDate && $hostDown ) {
            print FILE "$prevDate $hostDown\n";
            push( @xtics, "$prevDate" );
            push( @ytics, "$hostDown" );
            $finalDate = $prevDate;
            if ( $table[$row][$statusCol] =~ $down ) {
                $hostDown = 1;
                $totalIncidents++;
            }
            else {
                $hostDown = 0;
            }
        }
        elsif ( $prevDate =~ $currentDate ) {
            if ( $table[$row][$statusCol] =~ $down ) {
                $hostDown++;
                $totalIncidents++;
            }
        }
        if ( $row + 1 == @table && $hostDown ) {
            print FILE "$currentDate $hostDown\n";
            push( @xtics, "$currentDate" );
            push( @ytics, "$hostDown" );
            $finalDate = $currentDate;
        }
        if ( $maxHostDown < $hostDown ) {
            $maxHostDown = $hostDown;
        }
        $prevDate = $currentDate;
    }
    close(FILE);
    my $xtics = "\"" . join( "\",\"", @xtics ) . "\"";
    my $ytics = join( ",", @ytics );
    if (
        $self->runPlotter(
            $filename,  $maxHostDown, $table[0][$dateCol],
            $finalDate, $routerName,  $interval,
            $xtics,     $ytics,       $plotter,
            $plotOutput
        )
      )
    {
        return $totalIncidents;
    }
    return 0;
}

sub printRows {
    my ( $self, $columns, $table, $title ) = @_;

    print "<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">\n";
    print "<tr>";
    foreach my $i (@$title) {
        print "<td align=\"center\">$i</td>";
    }
    print "</tr>\n";
    for ( my $row = 0 ; $row < @$table ; $row++ ) {
        print "<tr>";
        for ( my $col = 0 ; $col < $columns ; $col++ ) {
            if ( $$table[$row][$col] =~ /down/ ) {
                print "<td><font color=\"#FF0000\">"
                  . $$table[$row][$col]
                  . " </font></td>";
            }
            elsif ( $$table[$row][$col] =~ /up/ ) {
                print "<td><font color=\"#66FF33\">"
                  . $$table[$row][$col]
                  . " </font></td>";
            }
            else {
                print "<td><font color=\"#000000\">"
                  . $$table[$row][$col]
                  . " </font></td>";
            }
        }
        print "</tr>\n";
    }
    print "</table>\n";
}

sub fetchRows {
    my ( $self, $query ) = @_;
    my @table = ();

    my $columns = 0;
    my $cvar    = $self->db()->prepare($query);
    $cvar->execute();
    if ( $cvar->rows > 0 ) {
        my $rows = 0;
        while ( my @result = $cvar->fetchrow_array() ) {
            $columns = 0;
            foreach my $field (@result) {
                $table[$rows][ $columns++ ] = $field;
            }
            $rows++;
        }
    }
    $cvar->finish();
    return @table;
}

1;
